/*
 * SSD1306 128x64に 8x8のフォントを表示
 * 
 */
#ifndef _SSD1306SPI_H_
#define _SSD1306SPI_H_
#include <arduino.h>
#include <SPI.h>

#define OLED_WIDTH 128  // OLEDの幅(ピクセル数)
#define OLED_HEIGHT 64  // OLEDの高さ(ピクセル数)
#define OLED_PAGES 8    // OLEDのページ数(ピクセル数÷８)
#define OLED_CHAR_W 16
#define OLED_CHAR_H 8

#define MONOOLED_WHITE 1
#define MONOOLED_BLACK 0
#define MONOOLED_INVERSE 2

#ifndef min
#define min(a, b) (((a) < (b)) ? (a) : (b))
#endif

#ifndef _swap_int16_t
#define _swap_int16_t(a, b)                                                    \
  {                                                                            \
    int16_t t = a;                                                             \
    a = b;                                                                     \
    b = t;                                                                     \
  }
#endif

class SSD1306SPI
{
  private:
    uint8_t DisplayBuff[OLED_WIDTH * OLED_PAGES] ;   // 表示Framebuffer
    uint8_t SPIBuf[OLED_WIDTH] ;                    // SPI転送用バッファ
    int curX , curY ; // 文字表示位置 X,Y
    int fontColor ;

  public:
    void init(); // 初期化
    void cls(); // 画面クリア
    void scroll() ;
    void dispSwitch(bool sw) ;
    void disp7SEG(int offsetX , int offsetY , int num , uint8_t color = MONOOLED_WHITE , bool flagDisp = true) ;
    void Print(char *str); // 文字列表示
    void Println(char *str,bool flagScroll = false); // 文字列表示
    void locate(int x , int y) ;
    void setFontColor(uint8_t color = MONOOLED_WHITE) { fontColor = color ; }
    void Print(int x , int y , char *str); // 文字列表示
    void drawPixel(int x, int y, uint8_t color,bool flagDisp=true) ;
    void drawLine(int x0, int y0, int x1, int y1, uint8_t color = MONOOLED_WHITE , bool flagDisp = true) ;
    void fillRect(int x, int y, int w, int h, uint8_t color = MONOOLED_WHITE , bool flagDisp = true) ;
    void drawRect(int16_t x, int16_t y, int16_t w, int16_t h, uint8_t color = MONOOLED_WHITE , bool flagDisp = true) ;
    void drawCircle(int16_t x0, int16_t y0, int16_t r, uint8_t color = MONOOLED_WHITE , bool flagDisp = true) ;
    void fillCircle(int16_t x0, int16_t y0, int16_t r, uint8_t color = MONOOLED_WHITE , bool flagDisp = true) ;

  protected:
    void clsOLED() ;
    void DrowFont(uint8_t code , uint8_t x, uint8_t y, uint8_t color = MONOOLED_WHITE , bool flagDisp = true) ;

    
    void oledBitMap(int col, int row, int width, uint8_t *bitMap) ;
    void oledBitMap(int offsetX , int offsetY , int width,int hight, uint8_t *bitMap) ;
    void oledDispFrame( ) ;
    void oledSendCommand(uint8_t command); // OLEDに1バイトコマンドを送信
    void oledSendCommand2(uint8_t command1, uint8_t command2); // OLEDに2バイトコマンドを送信
    void oledSendCommand3(uint8_t command1, uint8_t command2,uint8_t command3); // OLEDに3バイトコマンドを送信
    void oledSendCommands(int commandLen, uint8_t commands[]); // OLEDに複数のコマンドを送信
} ;
#endif
